DATABASE_ENGINE = 'sqlite3'

SITE_ID = 1

SECRET_KEY = '52b700e3a9dc3f83a410df37777fefbc'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'grappelli_navbar',
    'grappelli',
    'django.contrib.admin',
]

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'apptemplates.Loader',
)

TEST_RUNNER = 'discover_runner.DiscoverRunner'
GRAPPELLI_NAVBAR = u'grappelli_navbar.test_navbar.Navbar'

ROOT_URLCONF = 'grappelli_navbar.test_urls'
STATIC_URL = '/static/'

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.request',
    'django.core.context_processors.media',
    'django.contrib.messages.context_processors.messages',
    'django.core.context_processors.static',
)



 #!/usr/bin/env python
 #-*- coding:utf-8 -*-

import sys

from django.conf import settings

from django import template

from classytags.helpers import InclusionTag


GRAPPELLI_NAVBAR = \
    getattr(settings, "GRAPPELLI_NAVBAR", u'grappelli_navbar.navbar.Navbar')

parts = GRAPPELLI_NAVBAR.split(".")
module = ".".join(parts[:-1])
__import__(module)
module = sys.modules[module]
Navbar = getattr(module, parts[-1])


def has_perms(request, params):
    if 'perm' in params:
        perms = [params['perm']]
    else:
        perms = params.get('perms', [])
    if perms:
        perms = [request.user.has_perm(p) for p in perms]
        return any(perms)
    return True


def get_children(Navbar, request):
    children = []
    for node in Navbar.nodes:
        if node.__class__.__name__.endswith("Node"):
            title, params = node.as_tuple()
        else:
            title, params = node

        if not has_perms(request, params):
            continue

        nodes = params.get("nodes", [])
        url = params.get('url')
        root = {'title': title, 'children': [], 'url': url}
        for node in nodes:
            if node.__class__.__name__.endswith("Node"):
                title, params = node.as_tuple()
            else:
                title, params = node

            url = params.get('url')
            node = {'title': title, 'url': url}
            if has_perms(request, params):
                root['children'].append(node)

        if root['children'] or root['url']:
            children.append(root)
    return children


class GrappelliNavbar(InclusionTag):
    name = u"grappelli_navbar"
    template = 'grappelli/navbar.html'

    def get_context(self, context):
        return {'children': get_children(Navbar, context['request'])}


register = template.Library()
register.tag(GrappelliNavbar)



#-*- coding:utf-8 -*-

from django.core.urlresolvers import reverse
from django.db.models.loading import get_model


class Node(object):
    pass


class CLNode(Node):
    def __init__(self, app, model, title=None, perms=['change']):
        self.app = app
        self.model = model
        self.title = title
        self.perms = perms

    def as_tuple(self):
        model = get_model(self.app, self.model)
        vnp = model._meta.verbose_name_plural
        if self.title is None:
            title = \
                self.model.title() + 's' if vnp == self.model + 's' else vnp
        else:
            title = self.title
        perms = ['%s.%s_%s' % (self.app, i, self.model) for i in self.perms]
        return title, {
            'perms': perms,
            'url': reverse('admin:%s_%s_changelist' % (self.app, self.model))}



# -*- coding: utf-8 -*-

from django.template import Template, RequestContext
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.test.client import Client
from django.test import TestCase

from grappelli_navbar.templatetags.grappelli_navbar import get_children, Navbar


class NavBarTestCase(TestCase):
    def setUp(self):
        login = 'root'
        email = 'root@local.host'
        password = 'pass123'
        self.user = User.objects.create_superuser(login, email, password)
        self.client = Client()
        self.client.login(username=login, password=password)

    def test_navbar_default(self):
        factory = RequestFactory()
        request = factory.get(reverse("admin:index"))
        request.user = self.user
        children = get_children(Navbar, request)

        self.assertEqual(len(children), 4)

        tpl = Template(u"{% load grappelli_navbar %}{% grappelli_navbar %}")
        self.assertTrue(len(tpl.render(RequestContext(request))) > 0)

    def test_navbar_admin_page(self):
        response = self.client.get(reverse("admin:index"))
        self.assertEqual(len(response.context['children']), 4)



from django.conf.urls.defaults import url, include, patterns
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^grappelli/', include('grappelli.urls')),
    (r'^admin/', include(admin.site.urls)),
)



#-*- coding:utf-8 -*-

from django.core.urlresolvers import reverse_lazy

from grappelli_navbar.nodes import CLNode


class Navbar(object):
    nodes = (
        ('Auth', {'nodes': (
            ('Users', {
                'url': reverse_lazy('admin:auth_user_changelist'),
                'perm': 'auth.change_user',
            }),
            ('Groups', {
                'url': reverse_lazy('admin:auth_group_changelist'),
                'perm': 'auth.change_group',
            }),
        )}),
        ('Sites',
         {'url': reverse_lazy('admin:sites_site_changelist')}),
        ('Nodes', {'nodes': (
            CLNode('auth', 'user'),
            CLNode('sites', 'site'),
        )}),
        CLNode('auth', 'user', u"Site users"),
    )
